/*!
 * Debugger for Snowplow v4.6.5 (http://bit.ly/sp-js)
 * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
 * Licensed under BSD-3-Clause
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.snowplowDebugger = {}));
})(this, (function (exports) { 'use strict';

  /******************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */

  function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
  }

  typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
  };

  /*!
   * Core functionality for Snowplow JavaScript trackers v4.6.5 (http://bit.ly/sp-js)
   * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * Licensed under BSD-3-Clause
   */

  var version$1 = "4.6.5";

  /*
   * Copyright (c) 2013 Kevin van Zonneveld (http://kvz.io)
   * and Contributors (http://phpjs.org/authors)
   *
   * Permission is hereby granted, free of charge, to any person obtaining a copy of
   * this software and associated documentation files (the "Software"), to deal in
   * the Software without restriction, including without limitation the rights to
   * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
   * of the Software, and to permit persons to whom the Software is furnished to do
   * so, subject to the following conditions:
   *
   * The above copyright notice and this permission notice shall be included in all
   * copies or substantial portions of the Software.
   *
   * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   * SOFTWARE.
   */
  /**
   * Decodes a url safe Base 64 encoded string
   * @remarks See: {@link http://tools.ietf.org/html/rfc4648#page-7}
   * @param data - String to decode
   * @returns The decoded string
   */
  function base64urldecode(data) {
      if (!data) {
          return data;
      }
      var padding = 4 - (data.length % 4);
      switch (padding) {
          case 2:
              data += '==';
              break;
          case 3:
              data += '=';
              break;
      }
      var b64Data = data.replace(/-/g, '+').replace(/_/g, '/');
      return base64decode(b64Data);
  }
  /**
   * Encodes a string into a url safe Base 64 encoded string
   * @remarks See: {@link http://tools.ietf.org/html/rfc4648#page-7}
   * @param data - String to encode
   * @returns The url safe Base 64 string
   */
  function base64urlencode(data) {
      if (!data) {
          return data;
      }
      var enc = base64encode(data);
      return enc.replace(/=/g, '').replace(/\+/g, '-').replace(/\//g, '_');
  }
  var b64 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
  /**
   * Encode string as base64.
   * Any type can be passed, but will be stringified
   *
   * @param data - string to encode
   * @returns base64-encoded string
   */
  function base64encode(data) {
      // discuss at: http://phpjs.org/functions/base64_encode/
      // original by: Tyler Akins (http://rumkin.com)
      // improved by: Bayron Guevara
      // improved by: Thunder.m
      // improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
      // improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
      // improved by: Rafał Kukawski (http://kukawski.pl)
      // bugfixed by: Pellentesque Malesuada
      // example 1: base64_encode('Kevin van Zonneveld');
      // returns 1: 'S2V2aW4gdmFuIFpvbm5ldmVsZA=='
      // example 2: base64_encode('a');
      // returns 2: 'YQ=='
      // example 3: base64_encode('✓ à la mode');
      // returns 3: '4pyTIMOgIGxhIG1vZGU='
      var o1, o2, o3, h1, h2, h3, h4, bits, i = 0, ac = 0;
      var tmp_arr = [];
      if (!data) {
          return data;
      }
      data = unescape(encodeURIComponent(data));
      do {
          // pack three octets into four hexets
          o1 = data.charCodeAt(i++);
          o2 = data.charCodeAt(i++);
          o3 = data.charCodeAt(i++);
          bits = (o1 << 16) | (o2 << 8) | o3;
          h1 = (bits >> 18) & 0x3f;
          h2 = (bits >> 12) & 0x3f;
          h3 = (bits >> 6) & 0x3f;
          h4 = bits & 0x3f;
          // use hexets to index into b64, and append result to encoded string
          tmp_arr[ac++] = b64.charAt(h1) + b64.charAt(h2) + b64.charAt(h3) + b64.charAt(h4);
      } while (i < data.length);
      var enc = tmp_arr.join('');
      var r = data.length % 3;
      return (r ? enc.slice(0, r - 3) : enc) + '==='.slice(r || 3);
  }
  /**
   * Decode base64 to string
   *
   * @param data - base64 to string
   * @returns decoded string
   */
  function base64decode(encodedData) {
      //  discuss at: http://locutus.io/php/base64_decode/
      // original by: Tyler Akins (http://rumkin.com)
      // improved by: Thunder.m
      // improved by: Kevin van Zonneveld (http://kvz.io)
      // improved by: Kevin van Zonneveld (http://kvz.io)
      //    input by: Aman Gupta
      //    input by: Brett Zamir (http://brett-zamir.me)
      // bugfixed by: Onno Marsman (https://twitter.com/onnomarsman)
      // bugfixed by: Pellentesque Malesuada
      // bugfixed by: Kevin van Zonneveld (http://kvz.io)
      // improved by: Indigo744
      //   example 1: base64_decode('S2V2aW4gdmFuIFpvbm5ldmVsZA==')
      //   returns 1: 'Kevin van Zonneveld'
      //   example 2: base64_decode('YQ==')
      //   returns 2: 'a'
      //   example 3: base64_decode('4pyTIMOgIGxhIG1vZGU=')
      //   returns 3: '✓ à la mode'
      // decodeUTF8string()
      // Internal function to decode properly UTF8 string
      // Adapted from Solution #1 at https://developer.mozilla.org/en-US/docs/Web/API/WindowBase64/Base64_encoding_and_decoding
      var decodeUTF8string = function (str) {
          // Going backwards: from bytestream, to percent-encoding, to original string.
          return decodeURIComponent(str
              .split('')
              .map(function (c) {
              return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
          })
              .join(''));
      };
      var o1, o2, o3, h1, h2, h3, h4, bits, i = 0, ac = 0, dec = '';
      var tmpArr = [];
      if (!encodedData) {
          return encodedData;
      }
      encodedData += '';
      do {
          // unpack four hexets into three octets using index points in b64
          h1 = b64.indexOf(encodedData.charAt(i++));
          h2 = b64.indexOf(encodedData.charAt(i++));
          h3 = b64.indexOf(encodedData.charAt(i++));
          h4 = b64.indexOf(encodedData.charAt(i++));
          bits = (h1 << 18) | (h2 << 12) | (h3 << 6) | h4;
          o1 = (bits >> 16) & 0xff;
          o2 = (bits >> 8) & 0xff;
          o3 = bits & 0xff;
          if (h3 === 64) {
              tmpArr[ac++] = String.fromCharCode(o1);
          }
          else if (h4 === 64) {
              tmpArr[ac++] = String.fromCharCode(o1, o2);
          }
          else {
              tmpArr[ac++] = String.fromCharCode(o1, o2, o3);
          }
      } while (i < encodedData.length);
      dec = tmpArr.join('');
      return decodeUTF8string(dec.replace(/\0+$/, ''));
  }
  /**
   * A helper to build a Snowplow request from a set of name-value pairs, provided using the add methods.
   * Will base64 encode JSON, if desired, on build
   *
   * @returns The request builder, with add and build methods
   */
  function payloadJsonProcessor(encodeBase64) {
      return function (payloadBuilder, jsonForProcessing, contextEntitiesForProcessing) {
          var add = function (json, keyIfEncoded, keyIfNotEncoded) {
              var str = JSON.stringify(json);
              if (encodeBase64) {
                  payloadBuilder.add(keyIfEncoded, base64urlencode(str));
              }
              else {
                  payloadBuilder.add(keyIfNotEncoded, str);
              }
          };
          var getContextFromPayload = function () {
              var payload = payloadBuilder.getPayload();
              if (encodeBase64 ? payload.cx : payload.co) {
                  return JSON.parse(encodeBase64 ? base64urldecode(payload.cx) : payload.co);
              }
              return undefined;
          };
          var combineContexts = function (originalContext, newContext) {
              var context = originalContext || getContextFromPayload();
              if (context) {
                  context.data = context.data.concat(newContext.data);
              }
              else {
                  context = newContext;
              }
              return context;
          };
          var context = undefined;
          for (var _i = 0, jsonForProcessing_1 = jsonForProcessing; _i < jsonForProcessing_1.length; _i++) {
              var json = jsonForProcessing_1[_i];
              if (json.keyIfEncoded === 'cx') {
                  context = combineContexts(context, json.json);
              }
              else {
                  add(json.json, json.keyIfEncoded, json.keyIfNotEncoded);
              }
          }
          jsonForProcessing.length = 0;
          if (contextEntitiesForProcessing.length) {
              var newContext = {
                  schema: 'iglu:com.snowplowanalytics.snowplow/contexts/jsonschema/1-0-0',
                  data: __spreadArray([], contextEntitiesForProcessing, true),
              };
              context = combineContexts(context, newContext);
              contextEntitiesForProcessing.length = 0;
          }
          if (context) {
              add(context, 'cx', 'co');
          }
      };
  }
  var LOG_LEVEL;
  (function (LOG_LEVEL) {
      LOG_LEVEL[LOG_LEVEL["none"] = 0] = "none";
      LOG_LEVEL[LOG_LEVEL["error"] = 1] = "error";
      LOG_LEVEL[LOG_LEVEL["warn"] = 2] = "warn";
      LOG_LEVEL[LOG_LEVEL["debug"] = 3] = "debug";
      LOG_LEVEL[LOG_LEVEL["info"] = 4] = "info";
  })(LOG_LEVEL || (LOG_LEVEL = {}));
  /**
   * Slices a schema into its composite parts. Useful for ruleset filtering.
   * @param input - A schema string
   * @returns The vendor, schema name, major, minor and patch information of a schema string
   */
  function getSchemaParts(input) {
      var re = new RegExp('^iglu:([a-zA-Z0-9-_.]+)/([a-zA-Z0-9-_]+)/jsonschema/([1-9][0-9]*)-(0|[1-9][0-9]*)-(0|[1-9][0-9]*)$');
      var matches = re.exec(input);
      if (matches !== null)
          return matches.slice(1, 6);
      return undefined;
  }

  /*
   * Copyright (c) 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * All rights reserved.
   *
   * Redistribution and use in source and binary forms, with or without
   * modification, are permitted provided that the following conditions are met:
   *
   * 1. Redistributions of source code must retain the above copyright notice, this
   *    list of conditions and the following disclaimer.
   *
   * 2. Redistributions in binary form must reproduce the above copyright notice,
   *    this list of conditions and the following disclaimer in the documentation
   *    and/or other materials provided with the distribution.
   *
   * 3. Neither the name of the copyright holder nor the names of its
   *    contributors may be used to endorse or promote products derived from
   *    this software without specific prior written permission.
   *
   * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
   * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
   * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   */
  var version = version$1;

  var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

  var randomColor = {exports: {}};

  (function (module, exports) {
  (function(root, factory) {

    // Support CommonJS
    {
      var randomColor = factory();

      // Support NodeJS & Component, which allow module.exports to be a function
      if (module && module.exports) {
        exports = module.exports = randomColor;
      }

      // Support CommonJS 1.1.1 spec
      exports.randomColor = randomColor;

    // Support AMD
    }

  }(commonjsGlobal, function() {

    // Seed to get repeatable colors
    var seed = null;

    // Shared color dictionary
    var colorDictionary = {};

    // Populate the color dictionary
    loadColorBounds();

    // check if a range is taken
    var colorRanges = [];

    var randomColor = function (options) {

      options = options || {};

      // Check if there is a seed and ensure it's an
      // integer. Otherwise, reset the seed value.
      if (options.seed !== undefined && options.seed !== null && options.seed === parseInt(options.seed, 10)) {
        seed = options.seed;

      // A string was passed as a seed
      } else if (typeof options.seed === 'string') {
        seed = stringToInteger(options.seed);

      // Something was passed as a seed but it wasn't an integer or string
      } else if (options.seed !== undefined && options.seed !== null) {
        throw new TypeError('The seed value must be an integer or string');

      // No seed, reset the value outside.
      } else {
        seed = null;
      }

      var H,S,B;

      // Check if we need to generate multiple colors
      if (options.count !== null && options.count !== undefined) {

        var totalColors = options.count,
            colors = [];
        // Value false at index i means the range i is not taken yet.
        for (var i = 0; i < options.count; i++) {
          colorRanges.push(false);
          }
        options.count = null;

        while (totalColors > colors.length) {

          var color = randomColor(options);

          if (seed !== null) {
            options.seed = seed;
          }

          colors.push(color);
        }

        options.count = totalColors;

        return colors;
      }

      // First we pick a hue (H)
      H = pickHue(options);

      // Then use H to determine saturation (S)
      S = pickSaturation(H, options);

      // Then use S and H to determine brightness (B).
      B = pickBrightness(H, S, options);

      // Then we return the HSB color in the desired format
      return setFormat([H,S,B], options);
    };

    function pickHue(options) {
      if (colorRanges.length > 0) {
        var hueRange = getRealHueRange(options.hue);

        var hue = randomWithin(hueRange);

        //Each of colorRanges.length ranges has a length equal approximatelly one step
        var step = (hueRange[1] - hueRange[0]) / colorRanges.length;

        var j = parseInt((hue - hueRange[0]) / step);

        //Check if the range j is taken
        if (colorRanges[j] === true) {
          j = (j + 2) % colorRanges.length;
        }
        else {
          colorRanges[j] = true;
             }

        var min = (hueRange[0] + j * step) % 359,
            max = (hueRange[0] + (j + 1) * step) % 359;

        hueRange = [min, max];

        hue = randomWithin(hueRange);

        if (hue < 0) {hue = 360 + hue;}
        return hue
      }
      else {
        var hueRange = getHueRange(options.hue);

        hue = randomWithin(hueRange);
        // Instead of storing red as two seperate ranges,
        // we group them, using negative numbers
        if (hue < 0) {
          hue = 360 + hue;
        }

        return hue;
      }
    }

    function pickSaturation (hue, options) {

      if (options.hue === 'monochrome') {
        return 0;
      }

      if (options.luminosity === 'random') {
        return randomWithin([0,100]);
      }

      var saturationRange = getSaturationRange(hue);

      var sMin = saturationRange[0],
          sMax = saturationRange[1];

      switch (options.luminosity) {

        case 'bright':
          sMin = 55;
          break;

        case 'dark':
          sMin = sMax - 10;
          break;

        case 'light':
          sMax = 55;
          break;
     }

      return randomWithin([sMin, sMax]);

    }

    function pickBrightness (H, S, options) {

      var bMin = getMinimumBrightness(H, S),
          bMax = 100;

      switch (options.luminosity) {

        case 'dark':
          bMax = bMin + 20;
          break;

        case 'light':
          bMin = (bMax + bMin)/2;
          break;

        case 'random':
          bMin = 0;
          bMax = 100;
          break;
      }

      return randomWithin([bMin, bMax]);
    }

    function setFormat (hsv, options) {

      switch (options.format) {

        case 'hsvArray':
          return hsv;

        case 'hslArray':
          return HSVtoHSL(hsv);

        case 'hsl':
          var hsl = HSVtoHSL(hsv);
          return 'hsl('+hsl[0]+', '+hsl[1]+'%, '+hsl[2]+'%)';

        case 'hsla':
          var hslColor = HSVtoHSL(hsv);
          var alpha = options.alpha || Math.random();
          return 'hsla('+hslColor[0]+', '+hslColor[1]+'%, '+hslColor[2]+'%, ' + alpha + ')';

        case 'rgbArray':
          return HSVtoRGB(hsv);

        case 'rgb':
          var rgb = HSVtoRGB(hsv);
          return 'rgb(' + rgb.join(', ') + ')';

        case 'rgba':
          var rgbColor = HSVtoRGB(hsv);
          var alpha = options.alpha || Math.random();
          return 'rgba(' + rgbColor.join(', ') + ', ' + alpha + ')';

        default:
          return HSVtoHex(hsv);
      }

    }

    function getMinimumBrightness(H, S) {

      var lowerBounds = getColorInfo(H).lowerBounds;

      for (var i = 0; i < lowerBounds.length - 1; i++) {

        var s1 = lowerBounds[i][0],
            v1 = lowerBounds[i][1];

        var s2 = lowerBounds[i+1][0],
            v2 = lowerBounds[i+1][1];

        if (S >= s1 && S <= s2) {

           var m = (v2 - v1)/(s2 - s1),
               b = v1 - m*s1;

           return m*S + b;
        }

      }

      return 0;
    }

    function getHueRange (colorInput) {

      if (typeof parseInt(colorInput) === 'number') {

        var number = parseInt(colorInput);

        if (number < 360 && number > 0) {
          return [number, number];
        }

      }

      if (typeof colorInput === 'string') {

        if (colorDictionary[colorInput]) {
          var color = colorDictionary[colorInput];
          if (color.hueRange) {return color.hueRange;}
        } else if (colorInput.match(/^#?([0-9A-F]{3}|[0-9A-F]{6})$/i)) {
          var hue = HexToHSB(colorInput)[0];
          return [ hue, hue ];
        }
      }

      return [0,360];

    }

    function getSaturationRange (hue) {
      return getColorInfo(hue).saturationRange;
    }

    function getColorInfo (hue) {

      // Maps red colors to make picking hue easier
      if (hue >= 334 && hue <= 360) {
        hue-= 360;
      }

      for (var colorName in colorDictionary) {
         var color = colorDictionary[colorName];
         if (color.hueRange &&
             hue >= color.hueRange[0] &&
             hue <= color.hueRange[1]) {
            return colorDictionary[colorName];
         }
      } return 'Color not found';
    }

    function randomWithin (range) {
      if (seed === null) {
        //generate random evenly destinct number from : https://martin.ankerl.com/2009/12/09/how-to-create-random-colors-programmatically/
        var golden_ratio = 0.618033988749895;
        var r=Math.random();
        r += golden_ratio;
        r %= 1;
        return Math.floor(range[0] + r*(range[1] + 1 - range[0]));
      } else {
        //Seeded random algorithm from http://indiegamr.com/generate-repeatable-random-numbers-in-js/
        var max = range[1] || 1;
        var min = range[0] || 0;
        seed = (seed * 9301 + 49297) % 233280;
        var rnd = seed / 233280.0;
        return Math.floor(min + rnd * (max - min));
  }
    }

    function HSVtoHex (hsv){

      var rgb = HSVtoRGB(hsv);

      function componentToHex(c) {
          var hex = c.toString(16);
          return hex.length == 1 ? '0' + hex : hex;
      }

      var hex = '#' + componentToHex(rgb[0]) + componentToHex(rgb[1]) + componentToHex(rgb[2]);

      return hex;

    }

    function defineColor (name, hueRange, lowerBounds) {

      var sMin = lowerBounds[0][0],
          sMax = lowerBounds[lowerBounds.length - 1][0],

          bMin = lowerBounds[lowerBounds.length - 1][1],
          bMax = lowerBounds[0][1];

      colorDictionary[name] = {
        hueRange: hueRange,
        lowerBounds: lowerBounds,
        saturationRange: [sMin, sMax],
        brightnessRange: [bMin, bMax]
      };

    }

    function loadColorBounds () {

      defineColor(
        'monochrome',
        null,
        [[0,0],[100,0]]
      );

      defineColor(
        'red',
        [-26,18],
        [[20,100],[30,92],[40,89],[50,85],[60,78],[70,70],[80,60],[90,55],[100,50]]
      );

      defineColor(
        'orange',
        [18,46],
        [[20,100],[30,93],[40,88],[50,86],[60,85],[70,70],[100,70]]
      );

      defineColor(
        'yellow',
        [46,62],
        [[25,100],[40,94],[50,89],[60,86],[70,84],[80,82],[90,80],[100,75]]
      );

      defineColor(
        'green',
        [62,178],
        [[30,100],[40,90],[50,85],[60,81],[70,74],[80,64],[90,50],[100,40]]
      );

      defineColor(
        'blue',
        [178, 257],
        [[20,100],[30,86],[40,80],[50,74],[60,60],[70,52],[80,44],[90,39],[100,35]]
      );

      defineColor(
        'purple',
        [257, 282],
        [[20,100],[30,87],[40,79],[50,70],[60,65],[70,59],[80,52],[90,45],[100,42]]
      );

      defineColor(
        'pink',
        [282, 334],
        [[20,100],[30,90],[40,86],[60,84],[80,80],[90,75],[100,73]]
      );

    }

    function HSVtoRGB (hsv) {

      // this doesn't work for the values of 0 and 360
      // here's the hacky fix
      var h = hsv[0];
      if (h === 0) {h = 1;}
      if (h === 360) {h = 359;}

      // Rebase the h,s,v values
      h = h/360;
      var s = hsv[1]/100,
          v = hsv[2]/100;

      var h_i = Math.floor(h*6),
        f = h * 6 - h_i,
        p = v * (1 - s),
        q = v * (1 - f*s),
        t = v * (1 - (1 - f)*s),
        r = 256,
        g = 256,
        b = 256;

      switch(h_i) {
        case 0: r = v; g = t; b = p;  break;
        case 1: r = q; g = v; b = p;  break;
        case 2: r = p; g = v; b = t;  break;
        case 3: r = p; g = q; b = v;  break;
        case 4: r = t; g = p; b = v;  break;
        case 5: r = v; g = p; b = q;  break;
      }

      var result = [Math.floor(r*255), Math.floor(g*255), Math.floor(b*255)];
      return result;
    }

    function HexToHSB (hex) {
      hex = hex.replace(/^#/, '');
      hex = hex.length === 3 ? hex.replace(/(.)/g, '$1$1') : hex;

      var red = parseInt(hex.substr(0, 2), 16) / 255,
            green = parseInt(hex.substr(2, 2), 16) / 255,
            blue = parseInt(hex.substr(4, 2), 16) / 255;

      var cMax = Math.max(red, green, blue),
            delta = cMax - Math.min(red, green, blue),
            saturation = cMax ? (delta / cMax) : 0;

      switch (cMax) {
        case red: return [ 60 * (((green - blue) / delta) % 6) || 0, saturation, cMax ];
        case green: return [ 60 * (((blue - red) / delta) + 2) || 0, saturation, cMax ];
        case blue: return [ 60 * (((red - green) / delta) + 4) || 0, saturation, cMax ];
      }
    }

    function HSVtoHSL (hsv) {
      var h = hsv[0],
        s = hsv[1]/100,
        v = hsv[2]/100,
        k = (2-s)*v;

      return [
        h,
        Math.round(s*v / (k<1 ? k : 2-k) * 10000) / 100,
        k/2 * 100
      ];
    }

    function stringToInteger (string) {
      var total = 0;
      for (var i = 0; i !== string.length; i++) {
        if (total >= Number.MAX_SAFE_INTEGER) break;
        total += string.charCodeAt(i);
      }
      return total
    }

    // get The range of given hue when options.count!=0
    function getRealHueRange(colorHue)
    { if (!isNaN(colorHue)) {
      var number = parseInt(colorHue);

      if (number < 360 && number > 0) {
        return getColorInfo(colorHue).hueRange
      }
    }
      else if (typeof colorHue === 'string') {

        if (colorDictionary[colorHue]) {
          var color = colorDictionary[colorHue];

          if (color.hueRange) {
            return color.hueRange
         }
      } else if (colorHue.match(/^#?([0-9A-F]{3}|[0-9A-F]{6})$/i)) {
          var hue = HexToHSB(colorHue)[0];
          return getColorInfo(hue).hueRange
      }
    }

      return [0,360]
  }
    return randomColor;
  }));
  }(randomColor, randomColor.exports));

  var randomcolor = randomColor.exports;

  /*
   * Copyright (c) 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * All rights reserved.
   *
   * Redistribution and use in source and binary forms, with or without
   * modification, are permitted provided that the following conditions are met:
   *
   * 1. Redistributions of source code must retain the above copyright notice, this
   *    list of conditions and the following disclaimer.
   *
   * 2. Redistributions in binary form must reproduce the above copyright notice,
   *    this list of conditions and the following disclaimer in the documentation
   *    and/or other materials provided with the distribution.
   *
   * 3. Neither the name of the copyright holder nor the names of its
   *    contributors may be used to endorse or promote products derived from
   *    this software without specific prior written permission.
   *
   * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
   * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
   * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   */
  /**
   * Adds advertisement tracking functions
   */
  function DebuggerPlugin(logLevel) {
      if (logLevel === void 0) { logLevel = LOG_LEVEL.debug; }
      var LOG;
      var tracker;
      var eventColour;
      var colours = {
          event: function () { return "color: White; background: ".concat(eventColour, "; font-weight: bold; padding: 1px 4px; border-radius: 2px;"); },
          snowplowPurple: 'color: White; background: #6638B8; font-weight: bold; padding: 1px 4px; border-radius: 2px;',
          activated: 'color: White; background: #9E62DD; font-weight: bold; padding: 1px 4px; border-radius: 2px;',
          payload: 'color: White; background: #3748B8; font-weight: bold; padding: 1px 4px; border-radius: 2px;',
          json: 'color: White; background: #388AB8; font-weight: bold; padding: 1px 4px; border-radius: 2px;',
          schema: 'color: White; background: #268047; font-weight: bold; padding: 1px 4px; border-radius: 2px;',
          schemaVersion: 'color: White; background: #80265F; font-weight: bold; padding: 1px 4px; border-radius: 2px;',
      };
      var debug = function (style, message, extra) {
          var _a = extra !== null && extra !== void 0 ? extra : [''], extraMessage = _a[0], rest = _a.slice(1);
          LOG.debug.apply(LOG, __spreadArray(["v".concat(version, " %c").concat(tracker.namespace, "%c %c%s") + extraMessage,
              colours.snowplowPurple,
              '',
              style,
              message], rest, false));
      };
      function jsonInterceptor(encodeBase64) {
          var log = function (jsonType, data) {
              var schemaParts = getSchemaParts(data['schema']);
              debug(colours.event(), 'Event', [
                  '%c%s%c%s%c%s\n%o',
                  colours.json,
                  "".concat(jsonType, ": ").concat(schemaParts ? schemaParts[1] : 'Unknown Schema'),
                  colours.schemaVersion,
                  schemaParts ? "Version: ".concat(schemaParts[2], "-").concat(schemaParts[3], "-").concat(schemaParts[4]) : 'Unknown Schema Version',
                  colours.schema,
                  schemaParts ? "Vendor: ".concat(schemaParts[0]) : 'Unknown Vendor',
                  data['data'],
              ]);
          };
          return function (payloadBuilder, jsonForProcessing, contextEntitiesForProcessing) {
              if (jsonForProcessing.length) {
                  var _loop_1 = function (json) {
                      var data = json.json['data'];
                      if (Array.isArray(data)) {
                          data.forEach(function (d) {
                              log(getJsonType(json), d);
                          });
                      }
                      else {
                          log(getJsonType(json), data);
                      }
                  };
                  for (var _i = 0, jsonForProcessing_1 = jsonForProcessing; _i < jsonForProcessing_1.length; _i++) {
                      var json = jsonForProcessing_1[_i];
                      _loop_1(json);
                  }
              }
              if (contextEntitiesForProcessing.length) {
                  for (var _a = 0, contextEntitiesForProcessing_1 = contextEntitiesForProcessing; _a < contextEntitiesForProcessing_1.length; _a++) {
                      var entity = contextEntitiesForProcessing_1[_a];
                      log('Context', entity);
                  }
              }
              return payloadJsonProcessor(encodeBase64)(payloadBuilder, jsonForProcessing, contextEntitiesForProcessing);
          };
      }
      return {
          logger: function (logger) {
              LOG = logger;
              logger.setLogLevel(logLevel);
          },
          activateBrowserPlugin: function (t) {
              tracker = t;
              debug(colours.activated, 'Tracker Activated');
          },
          beforeTrack: function (payloadBuilder) {
              eventColour = randomcolor({ luminosity: 'dark' });
              payloadBuilder.withJsonProcessor(jsonInterceptor(tracker.core.getBase64Encoding()));
              debug(colours.event(), 'Event', ['%c%s', colours.snowplowPurple, getEventType(payloadBuilder)]);
              payloadBuilder.build();
          },
          afterTrack: function (payload) {
              debug(colours.event(), 'Event', ['%c%s\n%o', colours.payload, 'Payload', payload]);
          },
      };
  }
  function getJsonType(json) {
      switch (json.keyIfEncoded) {
          case 'cx':
              return 'Context';
          case 'ue_px':
              return 'Self Describing';
          default:
              return "".concat(json.keyIfEncoded, ", ").concat(json.keyIfNotEncoded);
      }
  }
  function getEventType(payloadBuilder) {
      var payload = payloadBuilder.getPayload();
      switch (payload['e']) {
          case 'pv':
              return 'Page View';
          case 'pp':
              return 'Page Ping';
          case 'tr':
              return 'Ecommerce Transaction';
          case 'ti':
              return 'Ecommerce Transaction Item';
          case 'se':
              return 'Structured Event';
          case 'ue':
              return 'Self Describing';
          default:
              return typeof payload['e'] === 'string' ? payload['e'] : 'Invalid';
      }
  }

  exports.DebuggerPlugin = DebuggerPlugin;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=index.umd.js.map
